import 'package:action_mixin/action_mixin.dart';
import 'package:example/events/dismis_dialog_event.dart';
import 'package:flutter/material.dart';
import 'package:flutter_hooks/flutter_hooks.dart';
import 'package:flutter_stream_listener/flutter_stream_listener.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';
import 'package:state_notifier/state_notifier.dart';

import 'event_mixin.dart';
import 'events/loading_event.dart';
import 'events/show_snackbar_event.dart';

class ST extends StateNotifier<int> with EventMixin {
  ST() : super(0);

  void increA() async {
    state++;
    putEvent(LoadingEvent());
    await Future.delayed(Duration(seconds: 1));
    putEvent(DismisDialogEvent());
    state++;
    putEvent(ShowSnackbarEvent(content: 'content'));
  }

  @override
  void dispose() {
    // TODO: implement dispose
    super.dispose();
    eventDispose();
  }
}

final stProvider = StateNotifierProvider((ref) => ST());

void main() {
  runApp(ProviderScope(child: MyApp()));
}

class MyApp extends StatelessWidget {
  // This widget is the root of your application.

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: 'Flutter Demo',
      theme: ThemeData(
        primarySwatch: Colors.blue,
        visualDensity: VisualDensity.adaptivePlatformDensity,
      ),
      home: MyHomePage(title: 'Flutter Demo Home Page'),
    );
  }
}

class MyHomePage extends StatefulHookWidget {
  MyHomePage({Key key, this.title}) : super(key: key);
  final String title;

  @override
  _MyHomePageState createState() => _MyHomePageState();
}

mixin aa {
  void onData(EventBase event, BuildContext context);
}

class _MyHomePageState extends State<MyHomePage> with aa {
  final _scaffoldKey = GlobalKey<ScaffoldState>();

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        key: _scaffoldKey,
        appBar: AppBar(
          title: Text(widget.title),
        ),
        body: StreamListener(
          stream: useProvider(stProvider).events,
          onData: (event) => onData(event, context),
          child: Column(
            mainAxisAlignment: MainAxisAlignment.center,
            children: <Widget>[],
          ),
        ),
        floatingActionButton: FloatingActionButton(
          onPressed: () {
            context.read(stProvider).increA();
          },
          tooltip: 'Increment',
          child: Icon(Icons.add),
        ));
  }

  @override
  void dispose() {
    eventDispose();
  }

  @override
  void onData(EventBase event, BuildContext context) {
    if (event is LoadingEvent) {
      showDialog(
          context: context,
          builder: (ct) {
            return Center(
              child: CircularProgressIndicator(),
            );
          });
    }
    if (event is DismisDialogEvent) {
      Navigator.of(context, rootNavigator: true).pop();
    }
    if (event is ShowSnackbarEvent) {
      _scaffoldKey.currentState.showSnackBar(SnackBar(
        content: Text(event.content),
      ));
    }
  }
}
